/**************************
 * (C) 2013 Kelley Hecker *
 *  All rights reserved   *
 **************************/

// Stores caterpillar segments and direction. Adds and moves segments and 
// changes direction.

#include "Caterpillar.h"

using namespace std;

// Constructs a caterpillar at the given coordinates in the grid. possibleColors 
// is the number of possible segment colors for the caterpillar.
Caterpillar::Caterpillar(int headX, int headY, int possibleColors) {
	segments.push_back(GridPoint(headX, headY, 0));

	numPossibleColors = possibleColors;
	lastColor = 0;

	currentDirection = UP;
}

// Get all segments.
vector<GridPoint> Caterpillar::GetSegments() {
	return segments;
}

// Get segment at an index.
GridPoint Caterpillar::GetSegment(int index) {
	return segments[index];
}

// Get the current direction.
Direction Caterpillar::GetDirection() {
	return currentDirection;
}

// Add a segment.
void Caterpillar::AddSegment() {
	lastColor = (lastColor+1)%numPossibleColors;

	segments.push_back(GridPoint(lastTailPosition.x, lastTailPosition.y, lastColor));
}

// Move all the segments.
void Caterpillar::MoveSegments() {
	lastTailPosition.x = segments[segments.size()-1].x;
	lastTailPosition.y = segments[segments.size()-1].y;

	// Move each segment to the next segment's position.
	for (unsigned int n = segments.size() - 1; n > 0; n--)
	{
		segments[n].x = segments[n-1].x;
		segments[n].y = segments[n-1].y;
	}

	// Move the head in the current direction.
	switch (currentDirection)
	{
	case UP:
		segments[0].y++;
		break;
	case DOWN:
		segments[0].y--;
		break;
	case RIGHT:
		segments[0].x++;
		break;
	case LEFT:
		segments[0].x--;
		break;
	}
}

// Change the direction of the caterpillar.
void Caterpillar::ChangeDirection(Direction newDir) {
	if (newDir == currentDirection)
		return;

	// Don't allow going backwards.
	if (currentDirection == UP && newDir == DOWN)
		return;
	if (currentDirection == DOWN && newDir == UP)
		return;
	if (currentDirection == LEFT && newDir == RIGHT)
		return;
	if (currentDirection == RIGHT && newDir == LEFT)
		return;
	
	currentDirection = newDir;
}

// Reset the caterpillar segments and direction.
void Caterpillar::Reset(int headX, int headY) {
	segments.clear();
	segments.push_back(GridPoint(headX, headY, 0));
	
	lastColor = 0;
	currentDirection = UP;
}