/**************************
 * (C) 2013 Kelley Hecker *
 *  All rights reserved   *
 **************************/

// Contains dimensions of the grid and stores targets. Also 
// converts grid points to openGL points and assigns colors. 
// Checks collisions.

#ifndef GAME_VIEW_H
#define GAME_VIEW_H

#include <vector>
#include "Structures.h"

using namespace std;

class GameBoard {
private:
	vector<GridPoint> targets;
	vector<Color> segmentColors;
	vector<Color> targetColors;
	int pixelWidth, pixelHeight;
	int gridWidth, gridHeight;
	float cellWidth, cellHeight;

public:
	// Constructor
	GameBoard(int pWidth, int pHeight, int gWidth, int gHeight, Color segmentColors[], int numSegmentColors, Color targetColors[], int numTargetColors);
	
	// Accessors
	Color GetSegmentColorAtIndex(int index);
	Color GetTargetColorAtIndex(int index);

	// Modifiers
	void AddTarget();

	// OpenGL helper functions
	vector<Sprite> ConvertGridPointsToSprites(vector<GridPoint> caterpillar);
	vector<Sprite> ConvertTargetsToSprites();

	// Game logic helper functions
	bool CheckHitTarget(int x, int y);
	bool CheckHitWall(int x, int y);
	bool CheckHitSelf(vector<GridPoint> segments);
	void Reset();
};

#endif